Package: groHMM
Version: 1.47.0
Date: 2025-07-24
Title: GRO-seq Analysis Pipeline
Authors@R: c(person("Charles G.", "Danko", role = c("aut"),
                email="dankoc@gmail.com"),
            person("Minho", "Chae", role = c("aut"),
                email="minho.chae@gmail.com"),
            person("Andre", "Martins", role = c("ctb"),
                email="alm253@cornell.edu"),
            person("W. Lee", "Kraus", role = c("aut", "fnd"),
                email="lee.kraus@utsouthwestern.edu"),
	    person("Anusha", "Nagari", role = c("ctb"),
                email="anusha.nagari@utsouthwestern.edu"),
	    person("Tulip", "Nandu", role = c("cre", "ctb"),
                email="tulip.nandu@utsouthwestern.edu"),
	    person("Pariksheet", "Nanda", role = c("ctb"),
                email="pan79@pitt.edu",
		comment = c(ORCID = "0000-0001-9726-4552")))
Depends: R (>= 4.1.0), MASS, parallel, S4Vectors (>= 0.17.25), IRanges
        (>= 2.13.12), Seqinfo, GenomicRanges (>= 1.31.8),
        GenomicAlignments (>= 1.15.6), rtracklayer (>= 1.39.7)
Suggests: BiocStyle, GenomicFeatures, edgeR, org.Hs.eg.db,
        TxDb.Hsapiens.UCSC.hg19.knownGene
Description: A pipeline for the analysis of GRO-seq data.
URL: https://github.com/Kraus-Lab/groHMM
BugReports: https://github.com/Kraus-Lab/groHMM/issues
License: GPL-3
biocViews: Sequencing, Software
LazyLoad: yes
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/groHMM
git_branch: devel
git_last_commit: 1ac2709
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: yes
Packaged: 2026-05-18 22:17:17 UTC; biocbuild
Author: Charles G. Danko [aut],
  Minho Chae [aut],
  Andre Martins [ctb],
  W. Lee Kraus [aut, fnd],
  Anusha Nagari [ctb],
  Tulip Nandu [cre, ctb],
  Pariksheet Nanda [ctb] (ORCID: <https://orcid.org/0000-0001-9726-4552>)
Maintainer: Tulip Nandu <tulip.nandu@utsouthwestern.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 04:54:06 UTC; unix
